@@ -27,7 +27,7 @@ pub use crate::bridge::{Abi3Version, BridgeModel, PyO3, PyO3Crate};
 pub use crate::build_context::{BuildContext, BuiltWheelMetadata};
 pub use crate::build_options::{BuildOptions, CargoOptions, TargetTriple};
 pub use crate::cargo_toml::CargoToml;
-pub use crate::compile::{BuildArtifact, compile};
+pub use crate::compile::{BuildArtifact, CompileResult, compile};
 pub use crate::compression::{CompressionMethod, CompressionOptions};
 pub use crate::develop::{DevelopOptions, develop};
 #[cfg(feature = "schemars")]
