@@ -281,6 +281,17 @@ fn develop_uniffi_mixed() {
     }
 }
 
+#[test]
+fn develop_uniffi_multiple_crates() {
+    handle_result(develop::test_develop(
+        "test-crates/uniffi-multiple-crates",
+        None,
+        "develop-uniffi-multiple-crates",
+        false,
+        TestInstallBackend::Pip,
+    ));
+}
+
 #[test]
 fn develop_uniffi_multiple_binding_files() {
     if env::var("GITHUB_ACTIONS").is_ok() || which("uniffi-bindgen").is_ok() {
@@ -945,6 +956,7 @@ fn pyo3_mixed_include_exclude_sdist() {
                 "pyo3_mixed_include_exclude-2.1.3/Cargo.toml",
                 "pyo3_mixed_include_exclude-2.1.3/PKG-INFO",
                 "pyo3_mixed_include_exclude-2.1.3/README.md",
+                "pyo3_mixed_include_exclude-2.1.3/build.rs",
                 "pyo3_mixed_include_exclude-2.1.3/check_installed/check_installed.py",
                 "pyo3_mixed_include_exclude-2.1.3/pyo3_mixed_include_exclude/__init__.py",
                 "pyo3_mixed_include_exclude-2.1.3/pyo3_mixed_include_exclude/include_this_file",
@@ -975,6 +987,7 @@ fn pyo3_mixed_include_exclude_git_sdist_generator() {
                 "pyo3_mixed_include_exclude-2.1.3/Cargo.toml",
                 "pyo3_mixed_include_exclude-2.1.3/PKG-INFO",
                 "pyo3_mixed_include_exclude-2.1.3/README.md",
+                "pyo3_mixed_include_exclude-2.1.3/build.rs",
                 "pyo3_mixed_include_exclude-2.1.3/check_installed/check_installed.py",
                 "pyo3_mixed_include_exclude-2.1.3/pyo3_mixed_include_exclude/__init__.py",
                 "pyo3_mixed_include_exclude-2.1.3/pyo3_mixed_include_exclude/include_this_file",
@@ -1008,6 +1021,7 @@ fn pyo3_mixed_include_exclude_wheel_files() {
         "pyo3_mixed_include_exclude-2.1.3.dist-info/WHEEL",
         "pyo3_mixed_include_exclude-2.1.3.dist-info/entry_points.txt",
         "pyo3_mixed_include_exclude/__init__.py",
+        "pyo3_mixed_include_exclude/generated_info.txt",
         "pyo3_mixed_include_exclude/include_this_file",
         "pyo3_mixed_include_exclude/python_module/__init__.py",
         "pyo3_mixed_include_exclude/python_module/double.py",
